# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

MRCHECKERBOARDMASK_METADATA = Metadata(
    id="f1aa53a7b6030f421a6e8c4455b70ae4ec643303.boutiques",
    name="mrcheckerboardmask",
    package="mrtrix",
    container_image_tag="mrtrix3/mrtrix3:3.0.4",
)


_MrcheckerboardmaskConfigParamsDictNoTag = typing.TypedDict('_MrcheckerboardmaskConfigParamsDictNoTag', {
    "key": str,
    "value": str,
})
MrcheckerboardmaskConfigParamsDictTagged = typing.TypedDict('MrcheckerboardmaskConfigParamsDictTagged', {
    "@type": typing.Literal["config"],
    "key": str,
    "value": str,
})
MrcheckerboardmaskConfigParamsDict = _MrcheckerboardmaskConfigParamsDictNoTag | MrcheckerboardmaskConfigParamsDictTagged


_MrcheckerboardmaskParamsDictNoTag = typing.TypedDict('_MrcheckerboardmaskParamsDictNoTag', {
    "tiles": typing.NotRequired[int | None],
    "invert": bool,
    "nan": bool,
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[MrcheckerboardmaskConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "input": InputPathType,
    "output": str,
})
MrcheckerboardmaskParamsDictTagged = typing.TypedDict('MrcheckerboardmaskParamsDictTagged', {
    "@type": typing.Literal["mrtrix/mrcheckerboardmask"],
    "tiles": typing.NotRequired[int | None],
    "invert": bool,
    "nan": bool,
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[MrcheckerboardmaskConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "input": InputPathType,
    "output": str,
})
MrcheckerboardmaskParamsDict = _MrcheckerboardmaskParamsDictNoTag | MrcheckerboardmaskParamsDictTagged


def mrcheckerboardmask_config(
    key: str,
    value: str,
) -> MrcheckerboardmaskConfigParamsDictTagged:
    """
    Build parameters.
    
    Args:
        key: temporarily set the value of an MRtrix config file entry.
        value: temporarily set the value of an MRtrix config file entry.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "config",
        "key": key,
        "value": value,
    }
    return params


def mrcheckerboardmask_config_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `MrcheckerboardmaskConfigParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("key", None) is None:
        raise StyxValidationError("`key` must not be None")
    if not isinstance(params["key"], str):
        raise StyxValidationError(f'`key` has the wrong type: Received `{type(params.get("key", None))}` expected `str`')
    if params.get("value", None) is None:
        raise StyxValidationError("`value` must not be None")
    if not isinstance(params["value"], str):
        raise StyxValidationError(f'`value` has the wrong type: Received `{type(params.get("value", None))}` expected `str`')


def mrcheckerboardmask_config_cargs(
    params: MrcheckerboardmaskConfigParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-config")
    cargs.append(params.get("key", None))
    cargs.append(params.get("value", None))
    return cargs


class MrcheckerboardmaskOutputs(typing.NamedTuple):
    """
    Output object returned when calling `MrcheckerboardmaskParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output: OutputPathType
    """the output binary image mask."""


def mrcheckerboardmask_params(
    input_: InputPathType,
    output: str,
    tiles: int | None = None,
    invert: bool = False,
    nan: bool = False,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[MrcheckerboardmaskConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
) -> MrcheckerboardmaskParamsDictTagged:
    """
    Build parameters.
    
    Args:
        input_: the input image to be used as a template.
        output: the output binary image mask.
        tiles: specify the number of tiles in any direction.
        invert: invert output binary mask.
        nan: use NaN as the output zero value.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "mrtrix/mrcheckerboardmask",
        "invert": invert,
        "nan": nan,
        "info": info,
        "quiet": quiet,
        "debug": debug,
        "force": force,
        "help": help_,
        "version": version,
        "input": input_,
        "output": output,
    }
    if tiles is not None:
        params["tiles"] = tiles
    if nthreads is not None:
        params["nthreads"] = nthreads
    if config is not None:
        params["config"] = config
    return params


def mrcheckerboardmask_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `MrcheckerboardmaskParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("tiles", None) is not None:
        if not isinstance(params["tiles"], int):
            raise StyxValidationError(f'`tiles` has the wrong type: Received `{type(params.get("tiles", None))}` expected `int | None`')
    if params.get("invert", False) is None:
        raise StyxValidationError("`invert` must not be None")
    if not isinstance(params["invert"], bool):
        raise StyxValidationError(f'`invert` has the wrong type: Received `{type(params.get("invert", False))}` expected `bool`')
    if params.get("nan", False) is None:
        raise StyxValidationError("`nan` must not be None")
    if not isinstance(params["nan"], bool):
        raise StyxValidationError(f'`nan` has the wrong type: Received `{type(params.get("nan", False))}` expected `bool`')
    if params.get("info", False) is None:
        raise StyxValidationError("`info` must not be None")
    if not isinstance(params["info"], bool):
        raise StyxValidationError(f'`info` has the wrong type: Received `{type(params.get("info", False))}` expected `bool`')
    if params.get("quiet", False) is None:
        raise StyxValidationError("`quiet` must not be None")
    if not isinstance(params["quiet"], bool):
        raise StyxValidationError(f'`quiet` has the wrong type: Received `{type(params.get("quiet", False))}` expected `bool`')
    if params.get("debug", False) is None:
        raise StyxValidationError("`debug` must not be None")
    if not isinstance(params["debug"], bool):
        raise StyxValidationError(f'`debug` has the wrong type: Received `{type(params.get("debug", False))}` expected `bool`')
    if params.get("force", False) is None:
        raise StyxValidationError("`force` must not be None")
    if not isinstance(params["force"], bool):
        raise StyxValidationError(f'`force` has the wrong type: Received `{type(params.get("force", False))}` expected `bool`')
    if params.get("nthreads", None) is not None:
        if not isinstance(params["nthreads"], int):
            raise StyxValidationError(f'`nthreads` has the wrong type: Received `{type(params.get("nthreads", None))}` expected `int | None`')
    if params.get("config", None) is not None:
        if not isinstance(params["config"], list):
            raise StyxValidationError(f'`config` has the wrong type: Received `{type(params.get("config", None))}` expected `list[MrcheckerboardmaskConfigParamsDict] | None`')
        for e in params["config"]:
            mrcheckerboardmask_config_validate(e)
    if params.get("help", False) is None:
        raise StyxValidationError("`help` must not be None")
    if not isinstance(params["help"], bool):
        raise StyxValidationError(f'`help` has the wrong type: Received `{type(params.get("help", False))}` expected `bool`')
    if params.get("version", False) is None:
        raise StyxValidationError("`version` must not be None")
    if not isinstance(params["version"], bool):
        raise StyxValidationError(f'`version` has the wrong type: Received `{type(params.get("version", False))}` expected `bool`')
    if params.get("input", None) is None:
        raise StyxValidationError("`input` must not be None")
    if not isinstance(params["input"], (pathlib.Path, str)):
        raise StyxValidationError(f'`input` has the wrong type: Received `{type(params.get("input", None))}` expected `InputPathType`')
    if params.get("output", None) is None:
        raise StyxValidationError("`output` must not be None")
    if not isinstance(params["output"], str):
        raise StyxValidationError(f'`output` has the wrong type: Received `{type(params.get("output", None))}` expected `str`')


def mrcheckerboardmask_cargs(
    params: MrcheckerboardmaskParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("mrcheckerboardmask")
    if params.get("tiles", None) is not None:
        cargs.extend([
            "-tiles",
            str(params.get("tiles", None))
        ])
    if params.get("invert", False):
        cargs.append("-invert")
    if params.get("nan", False):
        cargs.append("-nan")
    if params.get("info", False):
        cargs.append("-info")
    if params.get("quiet", False):
        cargs.append("-quiet")
    if params.get("debug", False):
        cargs.append("-debug")
    if params.get("force", False):
        cargs.append("-force")
    if params.get("nthreads", None) is not None:
        cargs.extend([
            "-nthreads",
            str(params.get("nthreads", None))
        ])
    if params.get("config", None) is not None:
        cargs.extend([a for c in [mrcheckerboardmask_config_cargs(s, execution) for s in params.get("config", None)] for a in c])
    if params.get("help", False):
        cargs.append("-help")
    if params.get("version", False):
        cargs.append("-version")
    cargs.append(execution.input_file(params.get("input", None)))
    cargs.append(params.get("output", None))
    return cargs


def mrcheckerboardmask_outputs(
    params: MrcheckerboardmaskParamsDict,
    execution: Execution,
) -> MrcheckerboardmaskOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = MrcheckerboardmaskOutputs(
        root=execution.output_file("."),
        output=execution.output_file(params.get("output", None)),
    )
    return ret


def mrcheckerboardmask_execute(
    params: MrcheckerboardmaskParamsDict,
    runner: Runner | None = None,
) -> MrcheckerboardmaskOutputs:
    """
    mrcheckerboardmask
    
    Create bitwise checkerboard image.
    
    
    
    References:
    
    .
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MrcheckerboardmaskOutputs`).
    """
    mrcheckerboardmask_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(MRCHECKERBOARDMASK_METADATA)
    params = execution.params(params)
    cargs = mrcheckerboardmask_cargs(params, execution)
    ret = mrcheckerboardmask_outputs(params, execution)
    execution.run(cargs)
    return ret


def mrcheckerboardmask(
    input_: InputPathType,
    output: str,
    tiles: int | None = None,
    invert: bool = False,
    nan: bool = False,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[MrcheckerboardmaskConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
    runner: Runner | None = None,
) -> MrcheckerboardmaskOutputs:
    """
    mrcheckerboardmask
    
    Create bitwise checkerboard image.
    
    
    
    References:
    
    .
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        input_: the input image to be used as a template.
        output: the output binary image mask.
        tiles: specify the number of tiles in any direction.
        invert: invert output binary mask.
        nan: use NaN as the output zero value.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MrcheckerboardmaskOutputs`).
    """
    params = mrcheckerboardmask_params(
        tiles=tiles,
        invert=invert,
        nan=nan,
        info=info,
        quiet=quiet,
        debug=debug,
        force=force,
        nthreads=nthreads,
        config=config,
        help_=help_,
        version=version,
        input_=input_,
        output=output,
    )
    return mrcheckerboardmask_execute(params, runner)


__all__ = [
    "MRCHECKERBOARDMASK_METADATA",
    "MrcheckerboardmaskConfigParamsDict",
    "MrcheckerboardmaskConfigParamsDictTagged",
    "MrcheckerboardmaskOutputs",
    "MrcheckerboardmaskParamsDict",
    "MrcheckerboardmaskParamsDictTagged",
    "mrcheckerboardmask",
    "mrcheckerboardmask_config",
    "mrcheckerboardmask_execute",
    "mrcheckerboardmask_params",
]
