# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

DWIBIASCORRECT_METADATA = Metadata(
    id="b6c72322d3d00ba26448970257e9f0b213c34953.boutiques",
    name="dwibiascorrect",
    package="mrtrix",
    container_image_tag="mrtrix3/mrtrix3:3.0.4",
)


_DwibiascorrectFslgradParamsDictNoTag = typing.TypedDict('_DwibiascorrectFslgradParamsDictNoTag', {
    "bvecs": InputPathType,
    "bvals": InputPathType,
})
DwibiascorrectFslgradParamsDictTagged = typing.TypedDict('DwibiascorrectFslgradParamsDictTagged', {
    "@type": typing.Literal["fslgrad"],
    "bvecs": InputPathType,
    "bvals": InputPathType,
})
DwibiascorrectFslgradParamsDict = _DwibiascorrectFslgradParamsDictNoTag | DwibiascorrectFslgradParamsDictTagged


_DwibiascorrectConfigParamsDictNoTag = typing.TypedDict('_DwibiascorrectConfigParamsDictNoTag', {
    "key": str,
    "value": str,
})
DwibiascorrectConfigParamsDictTagged = typing.TypedDict('DwibiascorrectConfigParamsDictTagged', {
    "@type": typing.Literal["config"],
    "key": str,
    "value": str,
})
DwibiascorrectConfigParamsDict = _DwibiascorrectConfigParamsDictNoTag | DwibiascorrectConfigParamsDictTagged


_DwibiascorrectParamsDictNoTag = typing.TypedDict('_DwibiascorrectParamsDictNoTag', {
    "algorithm": str,
    "input_image": InputPathType,
    "output_image": str,
    "grad": typing.NotRequired[InputPathType | None],
    "fslgrad": typing.NotRequired[DwibiascorrectFslgradParamsDict | None],
    "mask_image": typing.NotRequired[InputPathType | None],
    "bias_image": typing.NotRequired[InputPathType | None],
    "nocleanup": bool,
    "scratch_dir": typing.NotRequired[InputPathType | None],
    "continue_scratch_dir": typing.NotRequired[list[InputPathType] | None],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[float | None],
    "config": typing.NotRequired[list[DwibiascorrectConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "ants_b": typing.NotRequired[str | None],
    "ants_c": typing.NotRequired[str | None],
    "ants_s": typing.NotRequired[str | None],
})
DwibiascorrectParamsDictTagged = typing.TypedDict('DwibiascorrectParamsDictTagged', {
    "@type": typing.Literal["mrtrix/dwibiascorrect"],
    "algorithm": str,
    "input_image": InputPathType,
    "output_image": str,
    "grad": typing.NotRequired[InputPathType | None],
    "fslgrad": typing.NotRequired[DwibiascorrectFslgradParamsDict | None],
    "mask_image": typing.NotRequired[InputPathType | None],
    "bias_image": typing.NotRequired[InputPathType | None],
    "nocleanup": bool,
    "scratch_dir": typing.NotRequired[InputPathType | None],
    "continue_scratch_dir": typing.NotRequired[list[InputPathType] | None],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[float | None],
    "config": typing.NotRequired[list[DwibiascorrectConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "ants_b": typing.NotRequired[str | None],
    "ants_c": typing.NotRequired[str | None],
    "ants_s": typing.NotRequired[str | None],
})
DwibiascorrectParamsDict = _DwibiascorrectParamsDictNoTag | DwibiascorrectParamsDictTagged


def dwibiascorrect_fslgrad(
    bvecs: InputPathType,
    bvals: InputPathType,
) -> DwibiascorrectFslgradParamsDictTagged:
    """
    Build parameters.
    
    Args:
        bvecs: Provide the diffusion-weighted gradient scheme used in the\
            acquisition in FSL bvecs/bvals format files. If a diffusion gradient\
            scheme is present in the input image header, the data provided with\
            this option will be instead used.
        bvals: Provide the diffusion-weighted gradient scheme used in the\
            acquisition in FSL bvecs/bvals format files. If a diffusion gradient\
            scheme is present in the input image header, the data provided with\
            this option will be instead used.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "fslgrad",
        "bvecs": bvecs,
        "bvals": bvals,
    }
    return params


def dwibiascorrect_fslgrad_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `DwibiascorrectFslgradParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("bvecs", None) is None:
        raise StyxValidationError("`bvecs` must not be None")
    if not isinstance(params["bvecs"], (pathlib.Path, str)):
        raise StyxValidationError(f'`bvecs` has the wrong type: Received `{type(params.get("bvecs", None))}` expected `InputPathType`')
    if params.get("bvals", None) is None:
        raise StyxValidationError("`bvals` must not be None")
    if not isinstance(params["bvals"], (pathlib.Path, str)):
        raise StyxValidationError(f'`bvals` has the wrong type: Received `{type(params.get("bvals", None))}` expected `InputPathType`')


def dwibiascorrect_fslgrad_cargs(
    params: DwibiascorrectFslgradParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-fslgrad")
    cargs.append(execution.input_file(params.get("bvecs", None)))
    cargs.append(execution.input_file(params.get("bvals", None)))
    return cargs


def dwibiascorrect_config(
    key: str,
    value: str,
) -> DwibiascorrectConfigParamsDictTagged:
    """
    Build parameters.
    
    Args:
        key: temporarily set the value of an MRtrix config file entry.
        value: temporarily set the value of an MRtrix config file entry.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "config",
        "key": key,
        "value": value,
    }
    return params


def dwibiascorrect_config_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `DwibiascorrectConfigParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("key", None) is None:
        raise StyxValidationError("`key` must not be None")
    if not isinstance(params["key"], str):
        raise StyxValidationError(f'`key` has the wrong type: Received `{type(params.get("key", None))}` expected `str`')
    if params.get("value", None) is None:
        raise StyxValidationError("`value` must not be None")
    if not isinstance(params["value"], str):
        raise StyxValidationError(f'`value` has the wrong type: Received `{type(params.get("value", None))}` expected `str`')


def dwibiascorrect_config_cargs(
    params: DwibiascorrectConfigParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-config")
    cargs.append(params.get("key", None))
    cargs.append(params.get("value", None))
    return cargs


class DwibiascorrectOutputs(typing.NamedTuple):
    """
    Output object returned when calling `DwibiascorrectParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_image_file: OutputPathType
    """File containing the corrected image series"""
    bias_image_file: OutputPathType | None
    """File containing the estimated bias field"""


def dwibiascorrect_params(
    algorithm: str,
    input_image: InputPathType,
    output_image: str,
    grad: InputPathType | None = None,
    fslgrad: DwibiascorrectFslgradParamsDict | None = None,
    mask_image: InputPathType | None = None,
    bias_image: InputPathType | None = None,
    nocleanup: bool = False,
    scratch_dir: InputPathType | None = None,
    continue_scratch_dir: list[InputPathType] | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: float | None = None,
    config: list[DwibiascorrectConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
    ants_b: str | None = None,
    ants_c: str | None = None,
    ants_s: str | None = None,
) -> DwibiascorrectParamsDictTagged:
    """
    Build parameters.
    
    Args:
        algorithm: Select the algorithm to be used for bias correction. Options\
            are: ants, fsl.
        input_image: The input image series to be corrected.
        output_image: The output corrected image series.
        grad: Provide the diffusion gradient table in MRtrix format.
        fslgrad: Provide the diffusion-weighted gradient scheme used in the\
            acquisition in FSL bvecs/bvals format files. If a diffusion gradient\
            scheme is present in the input image header, the data provided with\
            this option will be instead used.
        mask_image: Manually provide a mask image for bias field estimation.
        bias_image: Output the estimated bias field.
        nocleanup: Do not delete intermediate files during script execution,\
            and do not delete scratch directory at script completion.
        scratch_dir: Manually specify the path in which to generate the scratch\
            directory.
        continue_scratch_dir: Continue the script from a previous execution;\
            must provide the scratch directory path.
        info: Display information messages.
        quiet: Do not display information messages or progress status.
        debug: Display debugging messages.
        force: Force overwrite of output files.
        nthreads: Use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: Display help information and exit.
        version: Display version information and exit.
        ants_b: N4BiasFieldCorrection option -b (initial mesh resolution in mm,\
            spline order).
        ants_c: N4BiasFieldCorrection option -c (number of iterations,\
            convergence threshold).
        ants_s: N4BiasFieldCorrection option -s (shrink-factor applied to\
            spatial dimensions).
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "mrtrix/dwibiascorrect",
        "algorithm": algorithm,
        "input_image": input_image,
        "output_image": output_image,
        "nocleanup": nocleanup,
        "info": info,
        "quiet": quiet,
        "debug": debug,
        "force": force,
        "help": help_,
        "version": version,
    }
    if grad is not None:
        params["grad"] = grad
    if fslgrad is not None:
        params["fslgrad"] = fslgrad
    if mask_image is not None:
        params["mask_image"] = mask_image
    if bias_image is not None:
        params["bias_image"] = bias_image
    if scratch_dir is not None:
        params["scratch_dir"] = scratch_dir
    if continue_scratch_dir is not None:
        params["continue_scratch_dir"] = continue_scratch_dir
    if nthreads is not None:
        params["nthreads"] = nthreads
    if config is not None:
        params["config"] = config
    if ants_b is not None:
        params["ants_b"] = ants_b
    if ants_c is not None:
        params["ants_c"] = ants_c
    if ants_s is not None:
        params["ants_s"] = ants_s
    return params


def dwibiascorrect_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `DwibiascorrectParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("algorithm", None) is None:
        raise StyxValidationError("`algorithm` must not be None")
    if not isinstance(params["algorithm"], str):
        raise StyxValidationError(f'`algorithm` has the wrong type: Received `{type(params.get("algorithm", None))}` expected `str`')
    if params.get("input_image", None) is None:
        raise StyxValidationError("`input_image` must not be None")
    if not isinstance(params["input_image"], (pathlib.Path, str)):
        raise StyxValidationError(f'`input_image` has the wrong type: Received `{type(params.get("input_image", None))}` expected `InputPathType`')
    if params.get("output_image", None) is None:
        raise StyxValidationError("`output_image` must not be None")
    if not isinstance(params["output_image"], str):
        raise StyxValidationError(f'`output_image` has the wrong type: Received `{type(params.get("output_image", None))}` expected `str`')
    if params.get("grad", None) is not None:
        if not isinstance(params["grad"], (pathlib.Path, str)):
            raise StyxValidationError(f'`grad` has the wrong type: Received `{type(params.get("grad", None))}` expected `InputPathType | None`')
    if params.get("fslgrad", None) is not None:
        dwibiascorrect_fslgrad_validate(params["fslgrad"])
    if params.get("mask_image", None) is not None:
        if not isinstance(params["mask_image"], (pathlib.Path, str)):
            raise StyxValidationError(f'`mask_image` has the wrong type: Received `{type(params.get("mask_image", None))}` expected `InputPathType | None`')
    if params.get("bias_image", None) is not None:
        if not isinstance(params["bias_image"], (pathlib.Path, str)):
            raise StyxValidationError(f'`bias_image` has the wrong type: Received `{type(params.get("bias_image", None))}` expected `InputPathType | None`')
    if params.get("nocleanup", False) is None:
        raise StyxValidationError("`nocleanup` must not be None")
    if not isinstance(params["nocleanup"], bool):
        raise StyxValidationError(f'`nocleanup` has the wrong type: Received `{type(params.get("nocleanup", False))}` expected `bool`')
    if params.get("scratch_dir", None) is not None:
        if not isinstance(params["scratch_dir"], (pathlib.Path, str)):
            raise StyxValidationError(f'`scratch_dir` has the wrong type: Received `{type(params.get("scratch_dir", None))}` expected `InputPathType | None`')
    if params.get("continue_scratch_dir", None) is not None:
        if not isinstance(params["continue_scratch_dir"], list):
            raise StyxValidationError(f'`continue_scratch_dir` has the wrong type: Received `{type(params.get("continue_scratch_dir", None))}` expected `list[InputPathType] | None`')
        if len(params["continue_scratch_dir"]) != 2:
            raise StyxValidationError("Parameter `continue_scratch_dir` must contain exactly 2 elements")
        for e in params["continue_scratch_dir"]:
            if not isinstance(e, (pathlib.Path, str)):
                raise StyxValidationError(f'`continue_scratch_dir` has the wrong type: Received `{type(params.get("continue_scratch_dir", None))}` expected `list[InputPathType] | None`')
    if params.get("info", False) is None:
        raise StyxValidationError("`info` must not be None")
    if not isinstance(params["info"], bool):
        raise StyxValidationError(f'`info` has the wrong type: Received `{type(params.get("info", False))}` expected `bool`')
    if params.get("quiet", False) is None:
        raise StyxValidationError("`quiet` must not be None")
    if not isinstance(params["quiet"], bool):
        raise StyxValidationError(f'`quiet` has the wrong type: Received `{type(params.get("quiet", False))}` expected `bool`')
    if params.get("debug", False) is None:
        raise StyxValidationError("`debug` must not be None")
    if not isinstance(params["debug"], bool):
        raise StyxValidationError(f'`debug` has the wrong type: Received `{type(params.get("debug", False))}` expected `bool`')
    if params.get("force", False) is None:
        raise StyxValidationError("`force` must not be None")
    if not isinstance(params["force"], bool):
        raise StyxValidationError(f'`force` has the wrong type: Received `{type(params.get("force", False))}` expected `bool`')
    if params.get("nthreads", None) is not None:
        if not isinstance(params["nthreads"], (float, int)):
            raise StyxValidationError(f'`nthreads` has the wrong type: Received `{type(params.get("nthreads", None))}` expected `float | None`')
    if params.get("config", None) is not None:
        if not isinstance(params["config"], list):
            raise StyxValidationError(f'`config` has the wrong type: Received `{type(params.get("config", None))}` expected `list[DwibiascorrectConfigParamsDict] | None`')
        for e in params["config"]:
            dwibiascorrect_config_validate(e)
    if params.get("help", False) is None:
        raise StyxValidationError("`help` must not be None")
    if not isinstance(params["help"], bool):
        raise StyxValidationError(f'`help` has the wrong type: Received `{type(params.get("help", False))}` expected `bool`')
    if params.get("version", False) is None:
        raise StyxValidationError("`version` must not be None")
    if not isinstance(params["version"], bool):
        raise StyxValidationError(f'`version` has the wrong type: Received `{type(params.get("version", False))}` expected `bool`')
    if params.get("ants_b", None) is not None:
        if not isinstance(params["ants_b"], str):
            raise StyxValidationError(f'`ants_b` has the wrong type: Received `{type(params.get("ants_b", None))}` expected `str | None`')
    if params.get("ants_c", None) is not None:
        if not isinstance(params["ants_c"], str):
            raise StyxValidationError(f'`ants_c` has the wrong type: Received `{type(params.get("ants_c", None))}` expected `str | None`')
    if params.get("ants_s", None) is not None:
        if not isinstance(params["ants_s"], str):
            raise StyxValidationError(f'`ants_s` has the wrong type: Received `{type(params.get("ants_s", None))}` expected `str | None`')


def dwibiascorrect_cargs(
    params: DwibiascorrectParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("dwibiascorrect")
    cargs.append(params.get("algorithm", None))
    cargs.append(execution.input_file(params.get("input_image", None)))
    cargs.append(params.get("output_image", None))
    if params.get("grad", None) is not None:
        cargs.extend([
            "-grad",
            execution.input_file(params.get("grad", None))
        ])
    if params.get("fslgrad", None) is not None:
        cargs.extend(dwibiascorrect_fslgrad_cargs(params.get("fslgrad", None), execution))
    if params.get("mask_image", None) is not None:
        cargs.extend([
            "-mask",
            execution.input_file(params.get("mask_image", None))
        ])
    if params.get("bias_image", None) is not None:
        cargs.extend([
            "-bias",
            execution.input_file(params.get("bias_image", None))
        ])
    if params.get("nocleanup", False):
        cargs.append("-nocleanup")
    if params.get("scratch_dir", None) is not None:
        cargs.extend([
            "-scratch",
            execution.input_file(params.get("scratch_dir", None))
        ])
    if params.get("continue_scratch_dir", None) is not None:
        cargs.extend([
            "-continue",
            *[execution.input_file(f) for f in params.get("continue_scratch_dir", None)]
        ])
    if params.get("info", False):
        cargs.append("-info")
    if params.get("quiet", False):
        cargs.append("-quiet")
    if params.get("debug", False):
        cargs.append("-debug")
    if params.get("force", False):
        cargs.append("-force")
    if params.get("nthreads", None) is not None:
        cargs.extend([
            "-nthreads",
            str(params.get("nthreads", None))
        ])
    if params.get("config", None) is not None:
        cargs.extend([a for c in [dwibiascorrect_config_cargs(s, execution) for s in params.get("config", None)] for a in c])
    if params.get("help", False):
        cargs.append("-help")
    if params.get("version", False):
        cargs.append("-version")
    if params.get("ants_b", None) is not None:
        cargs.extend([
            "-ants.b",
            params.get("ants_b", None)
        ])
    if params.get("ants_c", None) is not None:
        cargs.extend([
            "-ants.c",
            params.get("ants_c", None)
        ])
    if params.get("ants_s", None) is not None:
        cargs.extend([
            "-ants.s",
            params.get("ants_s", None)
        ])
    return cargs


def dwibiascorrect_outputs(
    params: DwibiascorrectParamsDict,
    execution: Execution,
) -> DwibiascorrectOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = DwibiascorrectOutputs(
        root=execution.output_file("."),
        output_image_file=execution.output_file(params.get("output_image", None)),
        bias_image_file=execution.output_file(pathlib.Path(params.get("bias_image", None)).name) if (params.get("bias_image") is not None) else None,
    )
    return ret


def dwibiascorrect_execute(
    params: DwibiascorrectParamsDict,
    runner: Runner | None = None,
) -> DwibiascorrectOutputs:
    """
    dwibiascorrect
    
    Perform B1 field inhomogeneity correction for a DWI volume series using
    either ANTs or FSL.
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `DwibiascorrectOutputs`).
    """
    dwibiascorrect_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(DWIBIASCORRECT_METADATA)
    params = execution.params(params)
    cargs = dwibiascorrect_cargs(params, execution)
    ret = dwibiascorrect_outputs(params, execution)
    execution.run(cargs)
    return ret


def dwibiascorrect(
    algorithm: str,
    input_image: InputPathType,
    output_image: str,
    grad: InputPathType | None = None,
    fslgrad: DwibiascorrectFslgradParamsDict | None = None,
    mask_image: InputPathType | None = None,
    bias_image: InputPathType | None = None,
    nocleanup: bool = False,
    scratch_dir: InputPathType | None = None,
    continue_scratch_dir: list[InputPathType] | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: float | None = None,
    config: list[DwibiascorrectConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
    ants_b: str | None = None,
    ants_c: str | None = None,
    ants_s: str | None = None,
    runner: Runner | None = None,
) -> DwibiascorrectOutputs:
    """
    dwibiascorrect
    
    Perform B1 field inhomogeneity correction for a DWI volume series using
    either ANTs or FSL.
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        algorithm: Select the algorithm to be used for bias correction. Options\
            are: ants, fsl.
        input_image: The input image series to be corrected.
        output_image: The output corrected image series.
        grad: Provide the diffusion gradient table in MRtrix format.
        fslgrad: Provide the diffusion-weighted gradient scheme used in the\
            acquisition in FSL bvecs/bvals format files. If a diffusion gradient\
            scheme is present in the input image header, the data provided with\
            this option will be instead used.
        mask_image: Manually provide a mask image for bias field estimation.
        bias_image: Output the estimated bias field.
        nocleanup: Do not delete intermediate files during script execution,\
            and do not delete scratch directory at script completion.
        scratch_dir: Manually specify the path in which to generate the scratch\
            directory.
        continue_scratch_dir: Continue the script from a previous execution;\
            must provide the scratch directory path.
        info: Display information messages.
        quiet: Do not display information messages or progress status.
        debug: Display debugging messages.
        force: Force overwrite of output files.
        nthreads: Use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: Display help information and exit.
        version: Display version information and exit.
        ants_b: N4BiasFieldCorrection option -b (initial mesh resolution in mm,\
            spline order).
        ants_c: N4BiasFieldCorrection option -c (number of iterations,\
            convergence threshold).
        ants_s: N4BiasFieldCorrection option -s (shrink-factor applied to\
            spatial dimensions).
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `DwibiascorrectOutputs`).
    """
    params = dwibiascorrect_params(
        algorithm=algorithm,
        input_image=input_image,
        output_image=output_image,
        grad=grad,
        fslgrad=fslgrad,
        mask_image=mask_image,
        bias_image=bias_image,
        nocleanup=nocleanup,
        scratch_dir=scratch_dir,
        continue_scratch_dir=continue_scratch_dir,
        info=info,
        quiet=quiet,
        debug=debug,
        force=force,
        nthreads=nthreads,
        config=config,
        help_=help_,
        version=version,
        ants_b=ants_b,
        ants_c=ants_c,
        ants_s=ants_s,
    )
    return dwibiascorrect_execute(params, runner)


__all__ = [
    "DWIBIASCORRECT_METADATA",
    "DwibiascorrectConfigParamsDict",
    "DwibiascorrectConfigParamsDictTagged",
    "DwibiascorrectFslgradParamsDict",
    "DwibiascorrectFslgradParamsDictTagged",
    "DwibiascorrectOutputs",
    "DwibiascorrectParamsDict",
    "DwibiascorrectParamsDictTagged",
    "dwibiascorrect",
    "dwibiascorrect_config",
    "dwibiascorrect_execute",
    "dwibiascorrect_fslgrad",
    "dwibiascorrect_params",
]
