# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

MRHISTOGRAM_METADATA = Metadata(
    id="5c07265a48110ef8900f48adf78d74320253b0c0.boutiques",
    name="mrhistogram",
    package="mrtrix",
    container_image_tag="mrtrix3/mrtrix3:3.0.4",
)


_MrhistogramConfigParamsDictNoTag = typing.TypedDict('_MrhistogramConfigParamsDictNoTag', {
    "key": str,
    "value": str,
})
MrhistogramConfigParamsDictTagged = typing.TypedDict('MrhistogramConfigParamsDictTagged', {
    "@type": typing.Literal["config"],
    "key": str,
    "value": str,
})
MrhistogramConfigParamsDict = _MrhistogramConfigParamsDictNoTag | MrhistogramConfigParamsDictTagged


_MrhistogramParamsDictNoTag = typing.TypedDict('_MrhistogramParamsDictNoTag', {
    "bins": typing.NotRequired[int | None],
    "template": typing.NotRequired[InputPathType | None],
    "mask": typing.NotRequired[InputPathType | None],
    "ignorezero": bool,
    "allvolumes": bool,
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[MrhistogramConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "image": InputPathType,
    "hist": str,
})
MrhistogramParamsDictTagged = typing.TypedDict('MrhistogramParamsDictTagged', {
    "@type": typing.Literal["mrtrix/mrhistogram"],
    "bins": typing.NotRequired[int | None],
    "template": typing.NotRequired[InputPathType | None],
    "mask": typing.NotRequired[InputPathType | None],
    "ignorezero": bool,
    "allvolumes": bool,
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[MrhistogramConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "image": InputPathType,
    "hist": str,
})
MrhistogramParamsDict = _MrhistogramParamsDictNoTag | MrhistogramParamsDictTagged


def mrhistogram_config(
    key: str,
    value: str,
) -> MrhistogramConfigParamsDictTagged:
    """
    Build parameters.
    
    Args:
        key: temporarily set the value of an MRtrix config file entry.
        value: temporarily set the value of an MRtrix config file entry.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "config",
        "key": key,
        "value": value,
    }
    return params


def mrhistogram_config_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `MrhistogramConfigParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("key", None) is None:
        raise StyxValidationError("`key` must not be None")
    if not isinstance(params["key"], str):
        raise StyxValidationError(f'`key` has the wrong type: Received `{type(params.get("key", None))}` expected `str`')
    if params.get("value", None) is None:
        raise StyxValidationError("`value` must not be None")
    if not isinstance(params["value"], str):
        raise StyxValidationError(f'`value` has the wrong type: Received `{type(params.get("value", None))}` expected `str`')


def mrhistogram_config_cargs(
    params: MrhistogramConfigParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-config")
    cargs.append(params.get("key", None))
    cargs.append(params.get("value", None))
    return cargs


class MrhistogramOutputs(typing.NamedTuple):
    """
    Output object returned when calling `MrhistogramParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    hist: OutputPathType
    """the output histogram file"""


def mrhistogram_params(
    image: InputPathType,
    hist: str,
    bins: int | None = None,
    template: InputPathType | None = None,
    mask: InputPathType | None = None,
    ignorezero: bool = False,
    allvolumes: bool = False,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[MrhistogramConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
) -> MrhistogramParamsDictTagged:
    """
    Build parameters.
    
    Args:
        image: the input image from which the histogram will be computed.
        hist: the output histogram file.
        bins: Manually set the number of bins to use to generate the histogram.
        template: Use an existing histogram file as the template for histogram\
            formation.
        mask: Calculate the histogram only within a mask image.
        ignorezero: ignore zero-valued data during histogram construction.
        allvolumes: generate one histogram across all image volumes, rather\
            than one per image volume.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "mrtrix/mrhistogram",
        "ignorezero": ignorezero,
        "allvolumes": allvolumes,
        "info": info,
        "quiet": quiet,
        "debug": debug,
        "force": force,
        "help": help_,
        "version": version,
        "image": image,
        "hist": hist,
    }
    if bins is not None:
        params["bins"] = bins
    if template is not None:
        params["template"] = template
    if mask is not None:
        params["mask"] = mask
    if nthreads is not None:
        params["nthreads"] = nthreads
    if config is not None:
        params["config"] = config
    return params


def mrhistogram_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `MrhistogramParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("bins", None) is not None:
        if not isinstance(params["bins"], int):
            raise StyxValidationError(f'`bins` has the wrong type: Received `{type(params.get("bins", None))}` expected `int | None`')
    if params.get("template", None) is not None:
        if not isinstance(params["template"], (pathlib.Path, str)):
            raise StyxValidationError(f'`template` has the wrong type: Received `{type(params.get("template", None))}` expected `InputPathType | None`')
    if params.get("mask", None) is not None:
        if not isinstance(params["mask"], (pathlib.Path, str)):
            raise StyxValidationError(f'`mask` has the wrong type: Received `{type(params.get("mask", None))}` expected `InputPathType | None`')
    if params.get("ignorezero", False) is None:
        raise StyxValidationError("`ignorezero` must not be None")
    if not isinstance(params["ignorezero"], bool):
        raise StyxValidationError(f'`ignorezero` has the wrong type: Received `{type(params.get("ignorezero", False))}` expected `bool`')
    if params.get("allvolumes", False) is None:
        raise StyxValidationError("`allvolumes` must not be None")
    if not isinstance(params["allvolumes"], bool):
        raise StyxValidationError(f'`allvolumes` has the wrong type: Received `{type(params.get("allvolumes", False))}` expected `bool`')
    if params.get("info", False) is None:
        raise StyxValidationError("`info` must not be None")
    if not isinstance(params["info"], bool):
        raise StyxValidationError(f'`info` has the wrong type: Received `{type(params.get("info", False))}` expected `bool`')
    if params.get("quiet", False) is None:
        raise StyxValidationError("`quiet` must not be None")
    if not isinstance(params["quiet"], bool):
        raise StyxValidationError(f'`quiet` has the wrong type: Received `{type(params.get("quiet", False))}` expected `bool`')
    if params.get("debug", False) is None:
        raise StyxValidationError("`debug` must not be None")
    if not isinstance(params["debug"], bool):
        raise StyxValidationError(f'`debug` has the wrong type: Received `{type(params.get("debug", False))}` expected `bool`')
    if params.get("force", False) is None:
        raise StyxValidationError("`force` must not be None")
    if not isinstance(params["force"], bool):
        raise StyxValidationError(f'`force` has the wrong type: Received `{type(params.get("force", False))}` expected `bool`')
    if params.get("nthreads", None) is not None:
        if not isinstance(params["nthreads"], int):
            raise StyxValidationError(f'`nthreads` has the wrong type: Received `{type(params.get("nthreads", None))}` expected `int | None`')
    if params.get("config", None) is not None:
        if not isinstance(params["config"], list):
            raise StyxValidationError(f'`config` has the wrong type: Received `{type(params.get("config", None))}` expected `list[MrhistogramConfigParamsDict] | None`')
        for e in params["config"]:
            mrhistogram_config_validate(e)
    if params.get("help", False) is None:
        raise StyxValidationError("`help` must not be None")
    if not isinstance(params["help"], bool):
        raise StyxValidationError(f'`help` has the wrong type: Received `{type(params.get("help", False))}` expected `bool`')
    if params.get("version", False) is None:
        raise StyxValidationError("`version` must not be None")
    if not isinstance(params["version"], bool):
        raise StyxValidationError(f'`version` has the wrong type: Received `{type(params.get("version", False))}` expected `bool`')
    if params.get("image", None) is None:
        raise StyxValidationError("`image` must not be None")
    if not isinstance(params["image"], (pathlib.Path, str)):
        raise StyxValidationError(f'`image` has the wrong type: Received `{type(params.get("image", None))}` expected `InputPathType`')
    if params.get("hist", None) is None:
        raise StyxValidationError("`hist` must not be None")
    if not isinstance(params["hist"], str):
        raise StyxValidationError(f'`hist` has the wrong type: Received `{type(params.get("hist", None))}` expected `str`')


def mrhistogram_cargs(
    params: MrhistogramParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("mrhistogram")
    if params.get("bins", None) is not None:
        cargs.extend([
            "-bins",
            str(params.get("bins", None))
        ])
    if params.get("template", None) is not None:
        cargs.extend([
            "-template",
            execution.input_file(params.get("template", None))
        ])
    if params.get("mask", None) is not None:
        cargs.extend([
            "-mask",
            execution.input_file(params.get("mask", None))
        ])
    if params.get("ignorezero", False):
        cargs.append("-ignorezero")
    if params.get("allvolumes", False):
        cargs.append("-allvolumes")
    if params.get("info", False):
        cargs.append("-info")
    if params.get("quiet", False):
        cargs.append("-quiet")
    if params.get("debug", False):
        cargs.append("-debug")
    if params.get("force", False):
        cargs.append("-force")
    if params.get("nthreads", None) is not None:
        cargs.extend([
            "-nthreads",
            str(params.get("nthreads", None))
        ])
    if params.get("config", None) is not None:
        cargs.extend([a for c in [mrhistogram_config_cargs(s, execution) for s in params.get("config", None)] for a in c])
    if params.get("help", False):
        cargs.append("-help")
    if params.get("version", False):
        cargs.append("-version")
    cargs.append(execution.input_file(params.get("image", None)))
    cargs.append(params.get("hist", None))
    return cargs


def mrhistogram_outputs(
    params: MrhistogramParamsDict,
    execution: Execution,
) -> MrhistogramOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = MrhistogramOutputs(
        root=execution.output_file("."),
        hist=execution.output_file(params.get("hist", None)),
    )
    return ret


def mrhistogram_execute(
    params: MrhistogramParamsDict,
    runner: Runner | None = None,
) -> MrhistogramOutputs:
    """
    mrhistogram
    
    Generate a histogram of image intensities.
    
    
    
    References:
    
    .
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MrhistogramOutputs`).
    """
    mrhistogram_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(MRHISTOGRAM_METADATA)
    params = execution.params(params)
    cargs = mrhistogram_cargs(params, execution)
    ret = mrhistogram_outputs(params, execution)
    execution.run(cargs)
    return ret


def mrhistogram(
    image: InputPathType,
    hist: str,
    bins: int | None = None,
    template: InputPathType | None = None,
    mask: InputPathType | None = None,
    ignorezero: bool = False,
    allvolumes: bool = False,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[MrhistogramConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
    runner: Runner | None = None,
) -> MrhistogramOutputs:
    """
    mrhistogram
    
    Generate a histogram of image intensities.
    
    
    
    References:
    
    .
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        image: the input image from which the histogram will be computed.
        hist: the output histogram file.
        bins: Manually set the number of bins to use to generate the histogram.
        template: Use an existing histogram file as the template for histogram\
            formation.
        mask: Calculate the histogram only within a mask image.
        ignorezero: ignore zero-valued data during histogram construction.
        allvolumes: generate one histogram across all image volumes, rather\
            than one per image volume.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MrhistogramOutputs`).
    """
    params = mrhistogram_params(
        bins=bins,
        template=template,
        mask=mask,
        ignorezero=ignorezero,
        allvolumes=allvolumes,
        info=info,
        quiet=quiet,
        debug=debug,
        force=force,
        nthreads=nthreads,
        config=config,
        help_=help_,
        version=version,
        image=image,
        hist=hist,
    )
    return mrhistogram_execute(params, runner)


__all__ = [
    "MRHISTOGRAM_METADATA",
    "MrhistogramConfigParamsDict",
    "MrhistogramConfigParamsDictTagged",
    "MrhistogramOutputs",
    "MrhistogramParamsDict",
    "MrhistogramParamsDictTagged",
    "mrhistogram",
    "mrhistogram_config",
    "mrhistogram_execute",
    "mrhistogram_params",
]
