"""
MRTrix3

MRtrix3 provides a set of tools to perform various types of diffusion MRI
analyses, from various forms of tractography through to next-generation
group-level analyses. It is designed with consistency, performance, and
stability in mind, and is freely available under an open-source license. It is
developed and maintained by a team of experts in the field, fostering an active
community of users from diverse backgrounds.

Author: MRTrix3 Developers

URL: https://www.mrtrix.org/
"""
# This file was auto generated by Styx.
# Do not edit this file directly.

from .afdconnectivity import *
from .amp2response import *
from .amp2sh import *
from .connectome2tck import *
from .connectomeedit import *
from .connectomestats import *
from .dcmedit import *
from .dcminfo import *
from .dirflip import *
from .dirgen import *
from .dirmerge import *
from .dirorder import *
from .dirsplit import *
from .dirstat import *
from .dwi2adc import *
from .dwi2fod import *
from .dwi2mask import *
from .dwi2response import *
from .dwi2tensor import *
from .dwibiascorrect import *
from .dwidenoise import *
from .dwiextract import *
from .dwigradcheck import *
from .fixel2peaks import *
from .fixel2sh import *
from .fixel2tsf import *
from .fixel2voxel import *
from .fixelcfestats import *
from .fixelconnectivity import *
from .fixelconvert import *
from .fixelcorrespondence import *
from .fixelcrop import *
from .fixelfilter import *
from .fixelreorient import *
from .fod2dec import *
from .fod2fixel import *
from .label2colour import *
from .label2mesh import *
from .labelconvert import *
from .labelstats import *
from .maskdump import *
from .maskfilter import *
from .mesh2voxel import *
from .meshconvert import *
from .meshfilter import *
from .mraverageheader import *
from .mrcalc import *
from .mrcat import *
from .mrcentroid import *
from .mrcheckerboardmask import *
from .mrclusterstats import *
from .mrcolour import *
from .mrconvert import *
from .mrdegibbs import *
from .mrdump import *
from .mredit import *
from .mrfilter import *
from .mrgrid import *
from .mrhistmatch import *
from .mrhistogram import *
from .mrinfo import *
from .mrmath import *
from .mrmetric import *
from .mrregister import *
from .mrstats import *
from .mrthreshold import *
from .mrtransform import *
from .mtnormalise import *
from .peaks2amp import *
from .peaks2fixel import *
from .responsemean import *
from .sh2amp import *
from .sh2peaks import *
from .sh2power import *
from .sh2response import *
from .shbasis import *
from .shconv import *
from .tck2connectome import *
from .tck2fixel import *
from .tckconvert import *
from .tckdfc import *
from .tckedit import *
from .tckgen import *
from .tckglobal import *
from .tckinfo import *
from .tckmap import *
from .tckresample import *
from .tcksample import *
from .tcksift import *
from .tcksift2 import *
from .tckstats import *
from .tcktransform import *
from .tensor2metric import *
from .transformcalc import *
from .transformcompose import *
from .transformconvert import *
from .tsfdivide import *
from .tsfinfo import *
from .tsfmult import *
from .tsfsmooth import *
from .tsfthreshold import *
from .tsfvalidate import *
from .v_5tt2gmwmi import *
from .v_5tt2vis import *
from .v_5ttcheck import *
from .v_5ttedit import *
from .v_5ttgen import *
from .vectorstats import *
from .voxel2fixel import *
from .voxel2mesh import *
from .warp2metric import *
from .warpconvert import *
from .warpcorrect import *
from .warpinit import *
from .warpinvert import *
from styxdefs import Runner as _Runner


def execute(
    params: dict,
    runner: _Runner | None = None,
) -> None:
    """
    Run a command in this package dynamically from a params object.
    
    Args:
        params: The parameters.
        runner: Command runner.
    """
    return {
        "mrtrix/5tt2gmwmi": v_5tt2gmwmi_execute,
        "mrtrix/5tt2vis": v_5tt2vis_execute,
        "mrtrix/5ttcheck": v_5ttcheck_execute,
        "mrtrix/5ttedit": v_5ttedit_execute,
        "mrtrix/5ttgen": v_5ttgen_execute,
        "mrtrix/afdconnectivity": afdconnectivity_execute,
        "mrtrix/amp2response": amp2response_execute,
        "mrtrix/amp2sh": amp2sh_execute,
        "mrtrix/connectome2tck": connectome2tck_execute,
        "mrtrix/connectomeedit": connectomeedit_execute,
        "mrtrix/connectomestats": connectomestats_execute,
        "mrtrix/dcmedit": dcmedit_execute,
        "mrtrix/dcminfo": dcminfo_execute,
        "mrtrix/dirflip": dirflip_execute,
        "mrtrix/dirgen": dirgen_execute,
        "mrtrix/dirmerge": dirmerge_execute,
        "mrtrix/dirorder": dirorder_execute,
        "mrtrix/dirsplit": dirsplit_execute,
        "mrtrix/dirstat": dirstat_execute,
        "mrtrix/dwi2adc": dwi2adc_execute,
        "mrtrix/dwi2fod": dwi2fod_execute,
        "mrtrix/dwi2mask": dwi2mask_execute,
        "mrtrix/dwi2response": dwi2response_execute,
        "mrtrix/dwi2tensor": dwi2tensor_execute,
        "mrtrix/dwibiascorrect": dwibiascorrect_execute,
        "mrtrix/dwidenoise": dwidenoise_execute,
        "mrtrix/dwiextract": dwiextract_execute,
        "mrtrix/dwigradcheck": dwigradcheck_execute,
        "mrtrix/fixel2peaks": fixel2peaks_execute,
        "mrtrix/fixel2sh": fixel2sh_execute,
        "mrtrix/fixel2tsf": fixel2tsf_execute,
        "mrtrix/fixel2voxel": fixel2voxel_execute,
        "mrtrix/fixelcfestats": fixelcfestats_execute,
        "mrtrix/fixelconnectivity": fixelconnectivity_execute,
        "mrtrix/fixelconvert": fixelconvert_execute,
        "mrtrix/fixelcorrespondence": fixelcorrespondence_execute,
        "mrtrix/fixelcrop": fixelcrop_execute,
        "mrtrix/fixelfilter": fixelfilter_execute,
        "mrtrix/fixelreorient": fixelreorient_execute,
        "mrtrix/fod2dec": fod2dec_execute,
        "mrtrix/fod2fixel": fod2fixel_execute,
        "mrtrix/label2colour": label2colour_execute,
        "mrtrix/label2mesh": label2mesh_execute,
        "mrtrix/labelconvert": labelconvert_execute,
        "mrtrix/labelstats": labelstats_execute,
        "mrtrix/maskdump": maskdump_execute,
        "mrtrix/maskfilter": maskfilter_execute,
        "mrtrix/mesh2voxel": mesh2voxel_execute,
        "mrtrix/meshconvert": meshconvert_execute,
        "mrtrix/meshfilter": meshfilter_execute,
        "mrtrix/mraverageheader": mraverageheader_execute,
        "mrtrix/mrcalc": mrcalc_execute,
        "mrtrix/mrcat": mrcat_execute,
        "mrtrix/mrcentroid": mrcentroid_execute,
        "mrtrix/mrcheckerboardmask": mrcheckerboardmask_execute,
        "mrtrix/mrclusterstats": mrclusterstats_execute,
        "mrtrix/mrcolour": mrcolour_execute,
        "mrtrix/mrconvert": mrconvert_execute,
        "mrtrix/mrdegibbs": mrdegibbs_execute,
        "mrtrix/mrdump": mrdump_execute,
        "mrtrix/mredit": mredit_execute,
        "mrtrix/mrfilter": mrfilter_execute,
        "mrtrix/mrgrid": mrgrid_execute,
        "mrtrix/mrhistmatch": mrhistmatch_execute,
        "mrtrix/mrhistogram": mrhistogram_execute,
        "mrtrix/mrinfo": mrinfo_execute,
        "mrtrix/mrmath": mrmath_execute,
        "mrtrix/mrmetric": mrmetric_execute,
        "mrtrix/mrregister": mrregister_execute,
        "mrtrix/mrstats": mrstats_execute,
        "mrtrix/mrthreshold": mrthreshold_execute,
        "mrtrix/mrtransform": mrtransform_execute,
        "mrtrix/mtnormalise": mtnormalise_execute,
        "mrtrix/peaks2amp": peaks2amp_execute,
        "mrtrix/peaks2fixel": peaks2fixel_execute,
        "mrtrix/responsemean": responsemean_execute,
        "mrtrix/sh2amp": sh2amp_execute,
        "mrtrix/sh2peaks": sh2peaks_execute,
        "mrtrix/sh2power": sh2power_execute,
        "mrtrix/sh2response": sh2response_execute,
        "mrtrix/shbasis": shbasis_execute,
        "mrtrix/shconv": shconv_execute,
        "mrtrix/tck2connectome": tck2connectome_execute,
        "mrtrix/tck2fixel": tck2fixel_execute,
        "mrtrix/tckconvert": tckconvert_execute,
        "mrtrix/tckdfc": tckdfc_execute,
        "mrtrix/tckedit": tckedit_execute,
        "mrtrix/tckgen": tckgen_execute,
        "mrtrix/tckglobal": tckglobal_execute,
        "mrtrix/tckinfo": tckinfo_execute,
        "mrtrix/tckmap": tckmap_execute,
        "mrtrix/tckresample": tckresample_execute,
        "mrtrix/tcksample": tcksample_execute,
        "mrtrix/tcksift": tcksift_execute,
        "mrtrix/tcksift2": tcksift2_execute,
        "mrtrix/tckstats": tckstats_execute,
        "mrtrix/tcktransform": tcktransform_execute,
        "mrtrix/tensor2metric": tensor2metric_execute,
        "mrtrix/transformcalc": transformcalc_execute,
        "mrtrix/transformcompose": transformcompose_execute,
        "mrtrix/transformconvert": transformconvert_execute,
        "mrtrix/tsfdivide": tsfdivide_execute,
        "mrtrix/tsfinfo": tsfinfo_execute,
        "mrtrix/tsfmult": tsfmult_execute,
        "mrtrix/tsfsmooth": tsfsmooth_execute,
        "mrtrix/tsfthreshold": tsfthreshold_execute,
        "mrtrix/tsfvalidate": tsfvalidate_execute,
        "mrtrix/vectorstats": vectorstats_execute,
        "mrtrix/voxel2fixel": voxel2fixel_execute,
        "mrtrix/voxel2mesh": voxel2mesh_execute,
        "mrtrix/warp2metric": warp2metric_execute,
        "mrtrix/warpconvert": warpconvert_execute,
        "mrtrix/warpcorrect": warpcorrect_execute,
        "mrtrix/warpinit": warpinit_execute,
        "mrtrix/warpinvert": warpinvert_execute,
    }[params["@type"]](params, runner)
