# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

DCMEDIT_METADATA = Metadata(
    id="8755d3035d88c63d3618eea7e9e40463fa8a47ee.boutiques",
    name="dcmedit",
    package="mrtrix",
    container_image_tag="mrtrix3/mrtrix3:3.0.4",
)


_DcmeditTagParamsDictNoTag = typing.TypedDict('_DcmeditTagParamsDictNoTag', {
    "group": str,
    "element": str,
    "newvalue": str,
})
DcmeditTagParamsDictTagged = typing.TypedDict('DcmeditTagParamsDictTagged', {
    "@type": typing.Literal["tag"],
    "group": str,
    "element": str,
    "newvalue": str,
})
DcmeditTagParamsDict = _DcmeditTagParamsDictNoTag | DcmeditTagParamsDictTagged


_DcmeditConfigParamsDictNoTag = typing.TypedDict('_DcmeditConfigParamsDictNoTag', {
    "key": str,
    "value": str,
})
DcmeditConfigParamsDictTagged = typing.TypedDict('DcmeditConfigParamsDictTagged', {
    "@type": typing.Literal["config"],
    "key": str,
    "value": str,
})
DcmeditConfigParamsDict = _DcmeditConfigParamsDictNoTag | DcmeditConfigParamsDictTagged


_DcmeditParamsDictNoTag = typing.TypedDict('_DcmeditParamsDictNoTag', {
    "anonymise": bool,
    "id": typing.NotRequired[str | None],
    "tag": typing.NotRequired[list[DcmeditTagParamsDict] | None],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[DcmeditConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "file": InputPathType,
})
DcmeditParamsDictTagged = typing.TypedDict('DcmeditParamsDictTagged', {
    "@type": typing.Literal["mrtrix/dcmedit"],
    "anonymise": bool,
    "id": typing.NotRequired[str | None],
    "tag": typing.NotRequired[list[DcmeditTagParamsDict] | None],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[DcmeditConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "file": InputPathType,
})
DcmeditParamsDict = _DcmeditParamsDictNoTag | DcmeditParamsDictTagged


def dcmedit_tag(
    group: str,
    element: str,
    newvalue: str,
) -> DcmeditTagParamsDictTagged:
    """
    Build parameters.
    
    Args:
        group: replace specific tag.
        element: replace specific tag.
        newvalue: replace specific tag.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "tag",
        "group": group,
        "element": element,
        "newvalue": newvalue,
    }
    return params


def dcmedit_tag_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `DcmeditTagParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("group", None) is None:
        raise StyxValidationError("`group` must not be None")
    if not isinstance(params["group"], str):
        raise StyxValidationError(f'`group` has the wrong type: Received `{type(params.get("group", None))}` expected `str`')
    if params.get("element", None) is None:
        raise StyxValidationError("`element` must not be None")
    if not isinstance(params["element"], str):
        raise StyxValidationError(f'`element` has the wrong type: Received `{type(params.get("element", None))}` expected `str`')
    if params.get("newvalue", None) is None:
        raise StyxValidationError("`newvalue` must not be None")
    if not isinstance(params["newvalue"], str):
        raise StyxValidationError(f'`newvalue` has the wrong type: Received `{type(params.get("newvalue", None))}` expected `str`')


def dcmedit_tag_cargs(
    params: DcmeditTagParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-tag")
    cargs.append(params.get("group", None))
    cargs.append(params.get("element", None))
    cargs.append(params.get("newvalue", None))
    return cargs


def dcmedit_config(
    key: str,
    value: str,
) -> DcmeditConfigParamsDictTagged:
    """
    Build parameters.
    
    Args:
        key: temporarily set the value of an MRtrix config file entry.
        value: temporarily set the value of an MRtrix config file entry.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "config",
        "key": key,
        "value": value,
    }
    return params


def dcmedit_config_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `DcmeditConfigParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("key", None) is None:
        raise StyxValidationError("`key` must not be None")
    if not isinstance(params["key"], str):
        raise StyxValidationError(f'`key` has the wrong type: Received `{type(params.get("key", None))}` expected `str`')
    if params.get("value", None) is None:
        raise StyxValidationError("`value` must not be None")
    if not isinstance(params["value"], str):
        raise StyxValidationError(f'`value` has the wrong type: Received `{type(params.get("value", None))}` expected `str`')


def dcmedit_config_cargs(
    params: DcmeditConfigParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-config")
    cargs.append(params.get("key", None))
    cargs.append(params.get("value", None))
    return cargs


class DcmeditOutputs(typing.NamedTuple):
    """
    Output object returned when calling `DcmeditParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def dcmedit_params(
    file: InputPathType,
    anonymise: bool = False,
    id_: str | None = None,
    tag: list[DcmeditTagParamsDict] | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[DcmeditConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
) -> DcmeditParamsDictTagged:
    """
    Build parameters.
    
    Args:
        file: the DICOM file to be edited.
        anonymise: remove any identifiable information, by replacing the\
            following tags:\
            - any tag with Value Representation PN will be replaced with\
            'anonymous'\
            - tag (0010,0030) PatientBirthDate will be replaced with an empty\
            string\
            WARNING: there is no guarantee that this command will remove all\
            identiable information, since such information may be contained in\
            any number of private vendor-specific tags. You will need to\
            double-check the results independently if you need to ensure\
            anonymity.
        id_: replace all ID tags with string supplied. This consists of tags\
            (0010, 0020) PatientID and (0010, 1000) OtherPatientIDs.
        tag: replace specific tag.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "mrtrix/dcmedit",
        "anonymise": anonymise,
        "info": info,
        "quiet": quiet,
        "debug": debug,
        "force": force,
        "help": help_,
        "version": version,
        "file": file,
    }
    if id_ is not None:
        params["id"] = id_
    if tag is not None:
        params["tag"] = tag
    if nthreads is not None:
        params["nthreads"] = nthreads
    if config is not None:
        params["config"] = config
    return params


def dcmedit_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `DcmeditParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("anonymise", False) is None:
        raise StyxValidationError("`anonymise` must not be None")
    if not isinstance(params["anonymise"], bool):
        raise StyxValidationError(f'`anonymise` has the wrong type: Received `{type(params.get("anonymise", False))}` expected `bool`')
    if params.get("id", None) is not None:
        if not isinstance(params["id"], str):
            raise StyxValidationError(f'`id` has the wrong type: Received `{type(params.get("id", None))}` expected `str | None`')
    if params.get("tag", None) is not None:
        if not isinstance(params["tag"], list):
            raise StyxValidationError(f'`tag` has the wrong type: Received `{type(params.get("tag", None))}` expected `list[DcmeditTagParamsDict] | None`')
        for e in params["tag"]:
            dcmedit_tag_validate(e)
    if params.get("info", False) is None:
        raise StyxValidationError("`info` must not be None")
    if not isinstance(params["info"], bool):
        raise StyxValidationError(f'`info` has the wrong type: Received `{type(params.get("info", False))}` expected `bool`')
    if params.get("quiet", False) is None:
        raise StyxValidationError("`quiet` must not be None")
    if not isinstance(params["quiet"], bool):
        raise StyxValidationError(f'`quiet` has the wrong type: Received `{type(params.get("quiet", False))}` expected `bool`')
    if params.get("debug", False) is None:
        raise StyxValidationError("`debug` must not be None")
    if not isinstance(params["debug"], bool):
        raise StyxValidationError(f'`debug` has the wrong type: Received `{type(params.get("debug", False))}` expected `bool`')
    if params.get("force", False) is None:
        raise StyxValidationError("`force` must not be None")
    if not isinstance(params["force"], bool):
        raise StyxValidationError(f'`force` has the wrong type: Received `{type(params.get("force", False))}` expected `bool`')
    if params.get("nthreads", None) is not None:
        if not isinstance(params["nthreads"], int):
            raise StyxValidationError(f'`nthreads` has the wrong type: Received `{type(params.get("nthreads", None))}` expected `int | None`')
    if params.get("config", None) is not None:
        if not isinstance(params["config"], list):
            raise StyxValidationError(f'`config` has the wrong type: Received `{type(params.get("config", None))}` expected `list[DcmeditConfigParamsDict] | None`')
        for e in params["config"]:
            dcmedit_config_validate(e)
    if params.get("help", False) is None:
        raise StyxValidationError("`help` must not be None")
    if not isinstance(params["help"], bool):
        raise StyxValidationError(f'`help` has the wrong type: Received `{type(params.get("help", False))}` expected `bool`')
    if params.get("version", False) is None:
        raise StyxValidationError("`version` must not be None")
    if not isinstance(params["version"], bool):
        raise StyxValidationError(f'`version` has the wrong type: Received `{type(params.get("version", False))}` expected `bool`')
    if params.get("file", None) is None:
        raise StyxValidationError("`file` must not be None")
    if not isinstance(params["file"], (pathlib.Path, str)):
        raise StyxValidationError(f'`file` has the wrong type: Received `{type(params.get("file", None))}` expected `InputPathType`')


def dcmedit_cargs(
    params: DcmeditParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("dcmedit")
    if params.get("anonymise", False):
        cargs.append("-anonymise")
    if params.get("id", None) is not None:
        cargs.extend([
            "-id",
            params.get("id", None)
        ])
    if params.get("tag", None) is not None:
        cargs.extend([a for c in [dcmedit_tag_cargs(s, execution) for s in params.get("tag", None)] for a in c])
    if params.get("info", False):
        cargs.append("-info")
    if params.get("quiet", False):
        cargs.append("-quiet")
    if params.get("debug", False):
        cargs.append("-debug")
    if params.get("force", False):
        cargs.append("-force")
    if params.get("nthreads", None) is not None:
        cargs.extend([
            "-nthreads",
            str(params.get("nthreads", None))
        ])
    if params.get("config", None) is not None:
        cargs.extend([a for c in [dcmedit_config_cargs(s, execution) for s in params.get("config", None)] for a in c])
    if params.get("help", False):
        cargs.append("-help")
    if params.get("version", False):
        cargs.append("-version")
    cargs.append(execution.input_file(params.get("file", None)))
    return cargs


def dcmedit_outputs(
    params: DcmeditParamsDict,
    execution: Execution,
) -> DcmeditOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = DcmeditOutputs(
        root=execution.output_file("."),
    )
    return ret


def dcmedit_execute(
    params: DcmeditParamsDict,
    runner: Runner | None = None,
) -> DcmeditOutputs:
    """
    dcmedit
    
    Edit DICOM file in-place.
    
    Note that this command simply replaces the existing values without modifying
    the DICOM structure in any way. Replacement text will be truncated if it is
    too long to fit inside the existing tag.
    
    WARNING: this command will modify existing data! It is recommended to run
    this command on a copy of the original data set to avoid loss of data.
    
    References:
    
    .
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `DcmeditOutputs`).
    """
    dcmedit_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(DCMEDIT_METADATA)
    params = execution.params(params)
    cargs = dcmedit_cargs(params, execution)
    ret = dcmedit_outputs(params, execution)
    execution.run(cargs)
    return ret


def dcmedit(
    file: InputPathType,
    anonymise: bool = False,
    id_: str | None = None,
    tag: list[DcmeditTagParamsDict] | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[DcmeditConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
    runner: Runner | None = None,
) -> DcmeditOutputs:
    """
    dcmedit
    
    Edit DICOM file in-place.
    
    Note that this command simply replaces the existing values without modifying
    the DICOM structure in any way. Replacement text will be truncated if it is
    too long to fit inside the existing tag.
    
    WARNING: this command will modify existing data! It is recommended to run
    this command on a copy of the original data set to avoid loss of data.
    
    References:
    
    .
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        file: the DICOM file to be edited.
        anonymise: remove any identifiable information, by replacing the\
            following tags:\
            - any tag with Value Representation PN will be replaced with\
            'anonymous'\
            - tag (0010,0030) PatientBirthDate will be replaced with an empty\
            string\
            WARNING: there is no guarantee that this command will remove all\
            identiable information, since such information may be contained in\
            any number of private vendor-specific tags. You will need to\
            double-check the results independently if you need to ensure\
            anonymity.
        id_: replace all ID tags with string supplied. This consists of tags\
            (0010, 0020) PatientID and (0010, 1000) OtherPatientIDs.
        tag: replace specific tag.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `DcmeditOutputs`).
    """
    params = dcmedit_params(
        anonymise=anonymise,
        id_=id_,
        tag=tag,
        info=info,
        quiet=quiet,
        debug=debug,
        force=force,
        nthreads=nthreads,
        config=config,
        help_=help_,
        version=version,
        file=file,
    )
    return dcmedit_execute(params, runner)


__all__ = [
    "DCMEDIT_METADATA",
    "DcmeditConfigParamsDict",
    "DcmeditConfigParamsDictTagged",
    "DcmeditOutputs",
    "DcmeditParamsDict",
    "DcmeditParamsDictTagged",
    "DcmeditTagParamsDict",
    "DcmeditTagParamsDictTagged",
    "dcmedit",
    "dcmedit_config",
    "dcmedit_execute",
    "dcmedit_params",
    "dcmedit_tag",
]
