# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

TCKCONVERT_METADATA = Metadata(
    id="55a4f5691d675bdf7129499cb50a199b880d7c7a.boutiques",
    name="tckconvert",
    package="mrtrix",
    container_image_tag="mrtrix3/mrtrix3:3.0.4",
)


_TckconvertConfigParamsDictNoTag = typing.TypedDict('_TckconvertConfigParamsDictNoTag', {
    "key": str,
    "value": str,
})
TckconvertConfigParamsDictTagged = typing.TypedDict('TckconvertConfigParamsDictTagged', {
    "@type": typing.Literal["config"],
    "key": str,
    "value": str,
})
TckconvertConfigParamsDict = _TckconvertConfigParamsDictNoTag | TckconvertConfigParamsDictTagged


_TckconvertVariousStringParamsDictNoTag = typing.TypedDict('_TckconvertVariousStringParamsDictNoTag', {
    "obj": str,
})
TckconvertVariousStringParamsDictTagged = typing.TypedDict('TckconvertVariousStringParamsDictTagged', {
    "@type": typing.Literal["VariousString"],
    "obj": str,
})
TckconvertVariousStringParamsDict = _TckconvertVariousStringParamsDictNoTag | TckconvertVariousStringParamsDictTagged


_TckconvertVariousFileParamsDictNoTag = typing.TypedDict('_TckconvertVariousFileParamsDictNoTag', {
    "obj": InputPathType,
})
TckconvertVariousFileParamsDictTagged = typing.TypedDict('TckconvertVariousFileParamsDictTagged', {
    "@type": typing.Literal["VariousFile"],
    "obj": InputPathType,
})
TckconvertVariousFileParamsDict = _TckconvertVariousFileParamsDictNoTag | TckconvertVariousFileParamsDictTagged


_TckconvertParamsDictNoTag = typing.TypedDict('_TckconvertParamsDictNoTag', {
    "scanner2voxel": typing.NotRequired[InputPathType | None],
    "scanner2image": typing.NotRequired[InputPathType | None],
    "voxel2scanner": typing.NotRequired[InputPathType | None],
    "image2scanner": typing.NotRequired[InputPathType | None],
    "sides": typing.NotRequired[int | None],
    "increment": typing.NotRequired[int | None],
    "dec": bool,
    "radius": typing.NotRequired[float | None],
    "ascii": bool,
    "binary": bool,
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[TckconvertConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "input": typing.Union[TckconvertVariousStringParamsDictTagged, TckconvertVariousFileParamsDictTagged],
    "output": str,
})
TckconvertParamsDictTagged = typing.TypedDict('TckconvertParamsDictTagged', {
    "@type": typing.Literal["mrtrix/tckconvert"],
    "scanner2voxel": typing.NotRequired[InputPathType | None],
    "scanner2image": typing.NotRequired[InputPathType | None],
    "voxel2scanner": typing.NotRequired[InputPathType | None],
    "image2scanner": typing.NotRequired[InputPathType | None],
    "sides": typing.NotRequired[int | None],
    "increment": typing.NotRequired[int | None],
    "dec": bool,
    "radius": typing.NotRequired[float | None],
    "ascii": bool,
    "binary": bool,
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[TckconvertConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "input": typing.Union[TckconvertVariousStringParamsDictTagged, TckconvertVariousFileParamsDictTagged],
    "output": str,
})
TckconvertParamsDict = _TckconvertParamsDictNoTag | TckconvertParamsDictTagged


def tckconvert_input_cargs_dyn_fn(
    t: str,
) -> typing.Any:
    """
    Get build cargs function by command type.
    
    Args:
        t: Command type.
    Returns:
        Build cargs function.
    """
    return {
        "VariousString": tckconvert_various_string_cargs,
        "VariousFile": tckconvert_various_file_cargs,
    }.get(t)


def tckconvert_input_validate_dyn_fn(
    t: str,
) -> typing.Any:
    """
    Get validate params function by command type.
    
    Args:
        t: Command type.
    Returns:
        Validate params function.
    """
    return {
        "VariousString": tckconvert_various_string_validate,
        "VariousFile": tckconvert_various_file_validate,
    }.get(t)


def tckconvert_config(
    key: str,
    value: str,
) -> TckconvertConfigParamsDictTagged:
    """
    Build parameters.
    
    Args:
        key: temporarily set the value of an MRtrix config file entry.
        value: temporarily set the value of an MRtrix config file entry.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "config",
        "key": key,
        "value": value,
    }
    return params


def tckconvert_config_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `TckconvertConfigParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("key", None) is None:
        raise StyxValidationError("`key` must not be None")
    if not isinstance(params["key"], str):
        raise StyxValidationError(f'`key` has the wrong type: Received `{type(params.get("key", None))}` expected `str`')
    if params.get("value", None) is None:
        raise StyxValidationError("`value` must not be None")
    if not isinstance(params["value"], str):
        raise StyxValidationError(f'`value` has the wrong type: Received `{type(params.get("value", None))}` expected `str`')


def tckconvert_config_cargs(
    params: TckconvertConfigParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-config")
    cargs.append(params.get("key", None))
    cargs.append(params.get("value", None))
    return cargs


def tckconvert_various_string(
    obj: str,
) -> TckconvertVariousStringParamsDictTagged:
    """
    Build parameters.
    
    Args:
        obj: String object.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "VariousString",
        "obj": obj,
    }
    return params


def tckconvert_various_string_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `TckconvertVariousStringParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("obj", None) is None:
        raise StyxValidationError("`obj` must not be None")
    if not isinstance(params["obj"], str):
        raise StyxValidationError(f'`obj` has the wrong type: Received `{type(params.get("obj", None))}` expected `str`')


def tckconvert_various_string_cargs(
    params: TckconvertVariousStringParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append(params.get("obj", None))
    return cargs


def tckconvert_various_file(
    obj: InputPathType,
) -> TckconvertVariousFileParamsDictTagged:
    """
    Build parameters.
    
    Args:
        obj: File object.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "VariousFile",
        "obj": obj,
    }
    return params


def tckconvert_various_file_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `TckconvertVariousFileParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("obj", None) is None:
        raise StyxValidationError("`obj` must not be None")
    if not isinstance(params["obj"], (pathlib.Path, str)):
        raise StyxValidationError(f'`obj` has the wrong type: Received `{type(params.get("obj", None))}` expected `InputPathType`')


def tckconvert_various_file_cargs(
    params: TckconvertVariousFileParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append(execution.input_file(params.get("obj", None)))
    return cargs


class TckconvertOutputs(typing.NamedTuple):
    """
    Output object returned when calling `TckconvertParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output: OutputPathType
    """the output track file."""


def tckconvert_params(
    input_: typing.Union[TckconvertVariousStringParamsDictTagged, TckconvertVariousFileParamsDictTagged],
    output: str,
    scanner2voxel: InputPathType | None = None,
    scanner2image: InputPathType | None = None,
    voxel2scanner: InputPathType | None = None,
    image2scanner: InputPathType | None = None,
    sides: int | None = None,
    increment: int | None = None,
    dec: bool = False,
    radius: float | None = None,
    ascii_: bool = False,
    binary: bool = False,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[TckconvertConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
) -> TckconvertParamsDictTagged:
    """
    Build parameters.
    
    Args:
        input_: the input track file.
        output: the output track file.
        scanner2voxel: if specified, the properties of this image will be used\
            to convert track point positions from real (scanner) coordinates into\
            voxel coordinates.
        scanner2image: if specified, the properties of this image will be used\
            to convert track point positions from real (scanner) coordinates into\
            image coordinates (in mm).
        voxel2scanner: if specified, the properties of this image will be used\
            to convert track point positions from voxel coordinates into real\
            (scanner) coordinates.
        image2scanner: if specified, the properties of this image will be used\
            to convert track point positions from image coordinates (in mm) into\
            real (scanner) coordinates.
        sides: number of sides for streamlines.
        increment: generate streamline points at every (increment) points.
        dec: add DEC as a primvar.
        radius: radius of the streamlines.
        ascii_: write an ASCII VTK file (this is the default).
        binary: write a binary VTK file.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "mrtrix/tckconvert",
        "dec": dec,
        "ascii": ascii_,
        "binary": binary,
        "info": info,
        "quiet": quiet,
        "debug": debug,
        "force": force,
        "help": help_,
        "version": version,
        "input": input_,
        "output": output,
    }
    if scanner2voxel is not None:
        params["scanner2voxel"] = scanner2voxel
    if scanner2image is not None:
        params["scanner2image"] = scanner2image
    if voxel2scanner is not None:
        params["voxel2scanner"] = voxel2scanner
    if image2scanner is not None:
        params["image2scanner"] = image2scanner
    if sides is not None:
        params["sides"] = sides
    if increment is not None:
        params["increment"] = increment
    if radius is not None:
        params["radius"] = radius
    if nthreads is not None:
        params["nthreads"] = nthreads
    if config is not None:
        params["config"] = config
    return params


def tckconvert_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `TckconvertParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("scanner2voxel", None) is not None:
        if not isinstance(params["scanner2voxel"], (pathlib.Path, str)):
            raise StyxValidationError(f'`scanner2voxel` has the wrong type: Received `{type(params.get("scanner2voxel", None))}` expected `InputPathType | None`')
    if params.get("scanner2image", None) is not None:
        if not isinstance(params["scanner2image"], (pathlib.Path, str)):
            raise StyxValidationError(f'`scanner2image` has the wrong type: Received `{type(params.get("scanner2image", None))}` expected `InputPathType | None`')
    if params.get("voxel2scanner", None) is not None:
        if not isinstance(params["voxel2scanner"], (pathlib.Path, str)):
            raise StyxValidationError(f'`voxel2scanner` has the wrong type: Received `{type(params.get("voxel2scanner", None))}` expected `InputPathType | None`')
    if params.get("image2scanner", None) is not None:
        if not isinstance(params["image2scanner"], (pathlib.Path, str)):
            raise StyxValidationError(f'`image2scanner` has the wrong type: Received `{type(params.get("image2scanner", None))}` expected `InputPathType | None`')
    if params.get("sides", None) is not None:
        if not isinstance(params["sides"], int):
            raise StyxValidationError(f'`sides` has the wrong type: Received `{type(params.get("sides", None))}` expected `int | None`')
    if params.get("increment", None) is not None:
        if not isinstance(params["increment"], int):
            raise StyxValidationError(f'`increment` has the wrong type: Received `{type(params.get("increment", None))}` expected `int | None`')
    if params.get("dec", False) is None:
        raise StyxValidationError("`dec` must not be None")
    if not isinstance(params["dec"], bool):
        raise StyxValidationError(f'`dec` has the wrong type: Received `{type(params.get("dec", False))}` expected `bool`')
    if params.get("radius", None) is not None:
        if not isinstance(params["radius"], (float, int)):
            raise StyxValidationError(f'`radius` has the wrong type: Received `{type(params.get("radius", None))}` expected `float | None`')
    if params.get("ascii", False) is None:
        raise StyxValidationError("`ascii` must not be None")
    if not isinstance(params["ascii"], bool):
        raise StyxValidationError(f'`ascii` has the wrong type: Received `{type(params.get("ascii", False))}` expected `bool`')
    if params.get("binary", False) is None:
        raise StyxValidationError("`binary` must not be None")
    if not isinstance(params["binary"], bool):
        raise StyxValidationError(f'`binary` has the wrong type: Received `{type(params.get("binary", False))}` expected `bool`')
    if params.get("info", False) is None:
        raise StyxValidationError("`info` must not be None")
    if not isinstance(params["info"], bool):
        raise StyxValidationError(f'`info` has the wrong type: Received `{type(params.get("info", False))}` expected `bool`')
    if params.get("quiet", False) is None:
        raise StyxValidationError("`quiet` must not be None")
    if not isinstance(params["quiet"], bool):
        raise StyxValidationError(f'`quiet` has the wrong type: Received `{type(params.get("quiet", False))}` expected `bool`')
    if params.get("debug", False) is None:
        raise StyxValidationError("`debug` must not be None")
    if not isinstance(params["debug"], bool):
        raise StyxValidationError(f'`debug` has the wrong type: Received `{type(params.get("debug", False))}` expected `bool`')
    if params.get("force", False) is None:
        raise StyxValidationError("`force` must not be None")
    if not isinstance(params["force"], bool):
        raise StyxValidationError(f'`force` has the wrong type: Received `{type(params.get("force", False))}` expected `bool`')
    if params.get("nthreads", None) is not None:
        if not isinstance(params["nthreads"], int):
            raise StyxValidationError(f'`nthreads` has the wrong type: Received `{type(params.get("nthreads", None))}` expected `int | None`')
    if params.get("config", None) is not None:
        if not isinstance(params["config"], list):
            raise StyxValidationError(f'`config` has the wrong type: Received `{type(params.get("config", None))}` expected `list[TckconvertConfigParamsDict] | None`')
        for e in params["config"]:
            tckconvert_config_validate(e)
    if params.get("help", False) is None:
        raise StyxValidationError("`help` must not be None")
    if not isinstance(params["help"], bool):
        raise StyxValidationError(f'`help` has the wrong type: Received `{type(params.get("help", False))}` expected `bool`')
    if params.get("version", False) is None:
        raise StyxValidationError("`version` must not be None")
    if not isinstance(params["version"], bool):
        raise StyxValidationError(f'`version` has the wrong type: Received `{type(params.get("version", False))}` expected `bool`')
    if params.get("input", None) is None:
        raise StyxValidationError("`input` must not be None")
    if not isinstance(params["input"], dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params["input"])}\'')
    if "@type" not in params["input"]:
        raise StyxValidationError("Params object is missing `@type`")
    if params["input"]["@type"] not in ["VariousString", "VariousFile"]:
        raise StyxValidationError("Parameter `input`s `@type` must be one of [\"VariousString\", \"VariousFile\"]")
    tckconvert_input_validate_dyn_fn(params["input"]["@type"])(params["input"])
    if params.get("output", None) is None:
        raise StyxValidationError("`output` must not be None")
    if not isinstance(params["output"], str):
        raise StyxValidationError(f'`output` has the wrong type: Received `{type(params.get("output", None))}` expected `str`')


def tckconvert_cargs(
    params: TckconvertParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("tckconvert")
    if params.get("scanner2voxel", None) is not None:
        cargs.extend([
            "-scanner2voxel",
            execution.input_file(params.get("scanner2voxel", None))
        ])
    if params.get("scanner2image", None) is not None:
        cargs.extend([
            "-scanner2image",
            execution.input_file(params.get("scanner2image", None))
        ])
    if params.get("voxel2scanner", None) is not None:
        cargs.extend([
            "-voxel2scanner",
            execution.input_file(params.get("voxel2scanner", None))
        ])
    if params.get("image2scanner", None) is not None:
        cargs.extend([
            "-image2scanner",
            execution.input_file(params.get("image2scanner", None))
        ])
    if params.get("sides", None) is not None:
        cargs.extend([
            "-sides",
            str(params.get("sides", None))
        ])
    if params.get("increment", None) is not None:
        cargs.extend([
            "-increment",
            str(params.get("increment", None))
        ])
    if params.get("dec", False):
        cargs.append("-dec")
    if params.get("radius", None) is not None:
        cargs.extend([
            "-radius",
            str(params.get("radius", None))
        ])
    if params.get("ascii", False):
        cargs.append("-ascii")
    if params.get("binary", False):
        cargs.append("-binary")
    if params.get("info", False):
        cargs.append("-info")
    if params.get("quiet", False):
        cargs.append("-quiet")
    if params.get("debug", False):
        cargs.append("-debug")
    if params.get("force", False):
        cargs.append("-force")
    if params.get("nthreads", None) is not None:
        cargs.extend([
            "-nthreads",
            str(params.get("nthreads", None))
        ])
    if params.get("config", None) is not None:
        cargs.extend([a for c in [tckconvert_config_cargs(s, execution) for s in params.get("config", None)] for a in c])
    if params.get("help", False):
        cargs.append("-help")
    if params.get("version", False):
        cargs.append("-version")
    cargs.extend(tckconvert_input_cargs_dyn_fn(params.get("input", None)["@type"])(params.get("input", None), execution))
    cargs.append(params.get("output", None))
    return cargs


def tckconvert_outputs(
    params: TckconvertParamsDict,
    execution: Execution,
) -> TckconvertOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = TckconvertOutputs(
        root=execution.output_file("."),
        output=execution.output_file(params.get("output", None)),
    )
    return ret


def tckconvert_execute(
    params: TckconvertParamsDict,
    runner: Runner | None = None,
) -> TckconvertOutputs:
    """
    tckconvert
    
    Convert between different track file formats.
    
    The program currently supports MRtrix .tck files (input/output), ascii text
    files (input/output), VTK polydata files (input/output), and RenderMan RIB
    (export only).
    
    Note that ascii files will be stored with one streamline per numbered file.
    To support this, the command will use the multi-file numbering syntax, where
    square brackets denote the position of the numbering for the files, for
    example:
    
    $ tckconvert input.tck output-'[]'.txt
    
    will produce files named output-0000.txt, output-0001.txt, output-0002.txt,
    ...
    
    References:
    
    .
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `TckconvertOutputs`).
    """
    tckconvert_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(TCKCONVERT_METADATA)
    params = execution.params(params)
    cargs = tckconvert_cargs(params, execution)
    ret = tckconvert_outputs(params, execution)
    execution.run(cargs)
    return ret


def tckconvert(
    input_: typing.Union[TckconvertVariousStringParamsDictTagged, TckconvertVariousFileParamsDictTagged],
    output: str,
    scanner2voxel: InputPathType | None = None,
    scanner2image: InputPathType | None = None,
    voxel2scanner: InputPathType | None = None,
    image2scanner: InputPathType | None = None,
    sides: int | None = None,
    increment: int | None = None,
    dec: bool = False,
    radius: float | None = None,
    ascii_: bool = False,
    binary: bool = False,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[TckconvertConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
    runner: Runner | None = None,
) -> TckconvertOutputs:
    """
    tckconvert
    
    Convert between different track file formats.
    
    The program currently supports MRtrix .tck files (input/output), ascii text
    files (input/output), VTK polydata files (input/output), and RenderMan RIB
    (export only).
    
    Note that ascii files will be stored with one streamline per numbered file.
    To support this, the command will use the multi-file numbering syntax, where
    square brackets denote the position of the numbering for the files, for
    example:
    
    $ tckconvert input.tck output-'[]'.txt
    
    will produce files named output-0000.txt, output-0001.txt, output-0002.txt,
    ...
    
    References:
    
    .
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        input_: the input track file.
        output: the output track file.
        scanner2voxel: if specified, the properties of this image will be used\
            to convert track point positions from real (scanner) coordinates into\
            voxel coordinates.
        scanner2image: if specified, the properties of this image will be used\
            to convert track point positions from real (scanner) coordinates into\
            image coordinates (in mm).
        voxel2scanner: if specified, the properties of this image will be used\
            to convert track point positions from voxel coordinates into real\
            (scanner) coordinates.
        image2scanner: if specified, the properties of this image will be used\
            to convert track point positions from image coordinates (in mm) into\
            real (scanner) coordinates.
        sides: number of sides for streamlines.
        increment: generate streamline points at every (increment) points.
        dec: add DEC as a primvar.
        radius: radius of the streamlines.
        ascii_: write an ASCII VTK file (this is the default).
        binary: write a binary VTK file.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `TckconvertOutputs`).
    """
    params = tckconvert_params(
        scanner2voxel=scanner2voxel,
        scanner2image=scanner2image,
        voxel2scanner=voxel2scanner,
        image2scanner=image2scanner,
        sides=sides,
        increment=increment,
        dec=dec,
        radius=radius,
        ascii_=ascii_,
        binary=binary,
        info=info,
        quiet=quiet,
        debug=debug,
        force=force,
        nthreads=nthreads,
        config=config,
        help_=help_,
        version=version,
        input_=input_,
        output=output,
    )
    return tckconvert_execute(params, runner)


__all__ = [
    "TCKCONVERT_METADATA",
    "TckconvertConfigParamsDict",
    "TckconvertConfigParamsDictTagged",
    "TckconvertOutputs",
    "TckconvertParamsDict",
    "TckconvertParamsDictTagged",
    "TckconvertVariousFileParamsDict",
    "TckconvertVariousFileParamsDictTagged",
    "TckconvertVariousStringParamsDict",
    "TckconvertVariousStringParamsDictTagged",
    "tckconvert",
    "tckconvert_config",
    "tckconvert_execute",
    "tckconvert_params",
    "tckconvert_various_file",
    "tckconvert_various_string",
]
